SimulateEvolutionOfSymbiontCommunities = function(
  #######################################################
  ###### You can either provide a whole population ######
   pop=NULL # a named vector giving the headcount of each kind of community
            # the names must be a serie of 0 and 1 indicating the presence or absence of all kind of *possible* symbiont communities. Their can be some 0 headcount, but all possible symbiont community must be their.
            # exemple with three types of symbionts :pop=c('000'=50,'001'=15,'010'=0,'011'=0,'100'=5,'101'=30,'110'=0,'111'=900)
            # here NeF = 1000; with 900 females that have the three symbionts; 50 aposymbiontics; no individuals that have the symbiont 2 but not the symbiont 1, and so on.
  ##################################################
  ###### Or the arguments needed to create it ######
  ,Nsymbionts = nchar(names(pop)[1]) # the number of symbionts
  ,NeF = sum(pop) # the effective female population size
  ,freq = rep(0.5,Nsymbionts) # the initial frequency of the symbionts.
                              # The frequencies of each type of community of symbionts will be set to be the ones expected in the absence of interaction between symbionts.
                              # If you don't want this, then define the argument 'pop' and leave the arguments Nsymbionts, NeF, and freq to their default
                              # it will be reset if SetFitnessesToMaintainPolymorphism == TRUE
  
  ,Ngenerations = NULL # the number of generation of evolution to simulate
  ,SavePopAtGenerations = c(0,Ngenerations) # the generations at which the pop should be saved (the first generation is the generation 0)
  ,MT=rep(1,Nsymbionts) # the rates of sucessfull maternal   transmission [0;1]
  ,HT=rep(0,Nsymbionts) # the rates of sucessfull horizontal transmission [0;1]
  ,SetFitnessesToMaintainPolymorphism=FALSE # should the initial frequency of the symbionts (freq) be randomly drawn in uniform distributions [0.01;0.99] and the fitnesses set to values that is expected to maintain polymorphism
  ,W =pop-pop+1         # A named vector (Same names as 'pop') describing the fitnesses induced by each kind of symbionts community
                        # for each offspring produced, for any female, the probability to be the mother will be proportional to this number (W)
                        # it will be reset if SetFitnessesToMaintainPolymorphism == TRUE
  ,verbose=TRUE
){
  library(gtools)
  library(extraDistr)
  
  if(max(SavePopAtGenerations)>Ngenerations){stop('max(SavePopAtGenerations)>Ngenerations ...')}
  if(!length(unique(names(pop)))==length(names(pop))){stop("You have some duplicated names in 'pop' ;  length(unique(names(pop)))==length(names(pop)) should return TRUE")}
  if(!all(sort(unique(strsplit(paste0(names(pop),collapse = ''),'')[[1]]))==c('0','1'))){stop("names(pop) can only contain '0' and '1'")}
  
  Symb = permutations(2, Nsymbionts,repeats.allowed=TRUE)-1
  rownames(Symb) = apply(Symb,1,function(x)paste0(x,collapse = ''))
  Ncommunities = nrow(Symb)
  
  if(is.null(pop)){
    Freq = apply(Symb ,1 ,function(ss)prod(freq[ss==1])*prod(1-freq[ss==0]))
    pop = rmultinom(1,NeF,Freq)[,1] # round(Freq*NeF) ; if(sum(pop)>NeF){pop = pop - rmultinom(1,sum(pop)-NeF,pop)[,1]} ; if(sum(pop)<NeF){pop = pop + rmultinom(1,NeF-sum(pop),pop)[,1]}
  }else if(!(Nsymbionts == nchar(names(pop)[1]) & NeF == sum(pop))){stop("If you define 'pop', then you cannont define 'Nsymbionts' nor 'NeF'")}
  
  if(!length(unique(nchar(names(pop))))==1){stop("The names of the object 'pop' are not consistant\nlength(unique(nchar(names(pop)))) should be equal to one")}

  
  if(length(MT)==1){MT = rep(MT,Nsymbionts)}
  if(length(HT)==1){HT = rep(HT,Nsymbionts)}
  
  if(SetFitnessesToMaintainPolymorphism){
    WperSymbionts = -1 ; while (any(WperSymbionts<0)) {
      freq = runif(Nsymbionts,0.01,0.99)
      WperSymbionts = sapply(1:Nsymbionts, function(i)
        (1-freq[i])/(1/(1-(1-(1-1/(NeF))^(freq[i]*NeF*HT[i]))*(1-freq[i])/freq[i]+1-MT[i])-freq[i])
      )
    }
    Freq = apply(Symb ,1 ,function(ss)prod(freq[ss==1])*prod(1-freq[ss==0]))
    pop = rmultinom(1,NeF,Freq)[,1] # round(Freq*NeF) ; if(sum(pop)>NeF){pop = pop - rmultinom(1,sum(pop)-NeF,pop)[,1]} ; if(sum(pop)<NeF){pop = pop + rmultinom(1,NeF-sum(pop),pop)[,1]}
    
    W = sapply(strsplit(names(pop),''), function(community)prod(WperSymbionts[community=="1"]))
    names(W)=names(pop)
  }
  
  if(!Ncommunities==length(pop)){stop("The names of 'pop' suggest you have ",Nsymbionts," types of symbionts\nThis corresponds to ",Ncommunities," communities of symbionts. But length of your pop is ",length(pop)," instead of ",Ncommunities,
                                      "\nIt will work if for example you complete this command line:\npop=c(",paste(rownames(Symb),collapse = "'=,'"),"=)")}
  
  names_ = names(pop)
  pop = pop[rownames(Symb)]
  W   =   W[rownames(Symb)]
  
  # SymbHT: symbiont community after an HT event as a function of the recipient community
  SymbHT = NULL ; for(symb in 1:Nsymbionts){
    SymbHT[[symb]] = Symb ; SymbHT[[symb]][,symb] = 1 ; SymbHT[[symb]] = apply(SymbHT[[symb]], 1, function(x)paste0(x, collapse = '')) }
  SymbMT = NULL ; for(symb in 1:Nsymbionts){
    SymbMT[[symb]] = Symb ; SymbMT[[symb]][,symb] = 0 ; SymbMT[[symb]] = apply(SymbMT[[symb]], 1, function(x)paste0(x, collapse = '')) }
  
  Pop = list('pop' = pop, 'Symb'=Symb)
  PopHistory = matrix(NA,nrow = length(SavePopAtGenerations),ncol = length(pop),dimnames = list(paste0('G',SavePopAtGenerations),names(pop)))
  if(is.element(0,SavePopAtGenerations)){PopHistory[1,]=pop ; saving=1}else{saving=0}
  ###### SIMULATE THE EVOLUTION ######
  G=0 ; while(!any(Pop$pop==NeF) & !any(Pop$pop==0) & G<Ngenerations){G=G+1 # stop when all polymorphism has been lost
    ### Horizontal Transmission
    if(any(HT>0)){for(symb in 1:Nsymbionts){#poop = Pop$pop
      Nht = rpois(1,HT[symb]*sum(Pop$Symb[,symb]*Pop$pop))
      # (we don't care about donors because donating don't change your infection status)
      # for each transmission event, individuals have a probability 1/NeF of being the recipient
      # the nbr of transmission events one individual undergoes thus follows a binomial distribution of n=Nht and p = 1/NeF
      # The probability of receiving one time or more the considered symbiont is thus 1-(1-1/NeF)^Nht (see the 'Description of the model.docx' file)
      # The number of individuals receiving the symbiont one time or more follows a new binomial distribution of n = NeF and p = 1-(1-1/NeF)^Nht
      Nrecipients = rbinom(1,  NeF  , (1-(1-1/NeF)^Nht) )
      # There will thus be on average (1-(1-1/NeF)^Nht)*Nht recipient individuals
      recipients = rmvhyper(nn=1, n = Pop$pop, k=Nrecipients)[1,] ; names(recipients) = names(Pop$pop) # without replacement
      # some individuals don't have the same community anymore
      Pop$pop = Pop$pop - recipients # remove them from the current community
      for(r in 1:length(recipients)){
        Pop$pop[SymbHT[[symb]][names(recipients)[r]]] = Pop$pop[SymbHT[[symb]][names(recipients)[r]]] + recipients[r] # add them to the new community
      }
    }}
    ### Reproduction 
    meanW = sum(W*Pop$pop/NeF)
    Pop$pop = rmultinom(1,NeF,(W/meanW)*Pop$pop)[,1] # sample with replacement
    ### Failed vertical transfers
    if(any(MT<1)){for(symb in 1:Nsymbionts){
      NvT = rpois(1,sum(Pop$Symb[,symb]*Pop$pop)*(1-MT[symb]))
      if(sum(Pop$Symb[,symb]*Pop$pop)<NvT){NvT = sum(Pop$Symb[,symb]*Pop$pop)}
      MTFailure = rmvhyper(nn=1, n = Pop$pop*Pop$Symb[,symb], k=NvT)[1,] ; names(MTFailure) = names(Pop$pop) # without replacement
      Pop$pop = Pop$pop - MTFailure # remove them from the current community
      Pop$pop[SymbMT[[symb]][names(MTFailure)]] = Pop$pop[SymbMT[[symb]][names(MTFailure)]] + MTFailure # add them to the new community
    }}
    ### save current state
    if(is.element(G,SavePopAtGenerations)){saving=saving+1
      PopHistory[saving,]=Pop$pop
    }
  }
  if(G<Ngenerations&verbose){cat("\nSimulation stopped at generation ",G," because polymorphisme was lost. \n")}
  return(PopHistory[,names_])
}


MutualistsSymbionts_HelpMe = function(){
cat(
  "This function is provided without any guaranty for anything
If you need help, feel free to contact me (hugomh{arrobbasse}gmx[D0t]fr)

You now have the function SimulateEvolutionOfSymbiontCommunities loaded in your global R environment
Its arguments are : 

  #######################################################
  ###### You can either provide a whole population ######
   pop=NULL # a named vector giving the headcount of each kind of community
            # the names must be a serie of 0 and 1 indicating the presence or absence of all kind of *possible* symbiont communities. Their can be some 0 headcount, but all possible symbiont community must be their.
            # exemple with three types of symbionts :pop=c('000'=50,'001'=15,'010'=0,'011'=0,'100'=5,'101'=30,'110'=0,'111'=900)
            # here NeF = 1000; with 900 females that have the three symbionts; 50 aposymbiontics; no individuals that have the symbiont 2 but not the symbiont 1, and so on.
  ##################################################
  ###### Or the arguments needed to create it ######
  ,Nsymbionts = nchar(names(pop)[1]) # the number of symbionts
  ,NeF = sum(pop) # the effective female population size
  ,freq = runif(Nsymbionts,0.01,0.99) # the initial frequency of the symbionts.
                                      # The frequencies of each type of community of symbionts will be set to be the ones expected in the absence of interaction between symbionts.
                                      # If you don't want this, then define the argument 'pop' and leave the arguments Nsymbionts, NeF, and freq to their default
  ##################################################
  ,Ngenerations = NULL # the number of generation of evolution to simulate
  ,SavePopAtGenerations = c(0,Ngenerations) # the generations at which the pop should be saved (the first generation is the generation 0)
  ,SexRatio=0.5
  ,MT=rep(1,Nsymbionts) # the rates of sucessfull maternal   transmission [0;1]
  ,HT=rep(0,Nsymbionts) # the rates of sucessfull horizontal transmission [0;1]
  ,W =pop-pop+1         # A named vector (Same names as 'pop') describing the fitnesses induced by each kind of symbionts community ; for each offspring produced, for any female, the probability to be the mother will be proportional to this number (W)
  ,verbose=TRUE

  
  if you want to see the message again, execute
  MutualistsSymbionts_HelpMe()")
  }
MutualistsSymbionts_HelpMe()

####################################################################################################
# ##### Iff the function permutations in gtoolsis no more available, then simly uncomment this code
#
# permutations = function (n, r, v = 1:n, set = TRUE, repeats.allowed = FALSE) 
# {
#     if (mode(n) != "numeric" || length(n) != 1 || n < 1 || (n%%1) != 
#         0) 
#         stop("bad value of n")
#     if (mode(r) != "numeric" || length(r) != 1 || r < 1 || (r%%1) != 
#         0) 
#         stop("bad value of r")
#     if (!is.atomic(v) || length(v) < n) 
#         stop("v is either non-atomic or too short")
#     if ((r > n) & repeats.allowed == FALSE) 
#         stop("r > n and repeats.allowed=FALSE")
#     if (set) {
#         v <- unique(sort(v))
#         if (length(v) < n) 
#             stop("too few different elements")
#     }
#     v0 <- vector(mode(v), 0)
#     if (repeats.allowed) 
#         sub <- function(n, r, v) {
#             if (r == 1) 
#                 matrix(v, n, 1)
#             else if (n == 1) 
#                 matrix(v, 1, r)
#             else {
#                 inner <- Recall(n, r - 1, v)
#                 cbind(rep(v, rep(nrow(inner), n)), matrix(t(inner), 
#                   ncol = ncol(inner), nrow = nrow(inner) * n, 
#                   byrow = TRUE))
#             }
#         }
#     else sub <- function(n, r, v) {
#         if (r == 1) 
#             matrix(v, n, 1)
#         else if (n == 1) 
#             matrix(v, 1, r)
#         else {
#             X <- NULL
#             for (i in 1:n) X <- rbind(X, cbind(v[i], Recall(n - 
#                 1, r - 1, v[-i])))
#             X
#         }
#     }
#     sub(n, r, v[1:n])
# }
